// Define gulp tasks.
module.exports = function (gulp, plugins, options) {

  'use strict';

  // Processor for linting is assigned to options so it can be reused later.
  options.processors = [
    // Options are defined in .stylelintrc.yaml file.
    plugins.stylelint(options.stylelint),
    plugins.reporter(options.processors.reporter)
  ];

  // Post CSS options.
  options.postcssOptions = [
    plugins.autoprefixer({browsers: options.browserSupport}),
    plugins.mqpacker({sort: true})
  ];

  // Gulp error handler.
  if (options.onError) {
    options.onErrorCallback = function (err) {
      plugins.notify.onError({
        title: options.onError.title + err.plugin,
        message: options.onError.message,
        sound: options.onError.sound
      })(err);
      this.emit('end');
    };
  }

  // Defining gulp tasks.

  // @todo - consider should we implement this.
  // @todo - For some other stuff check https://www.drupal.org/docs/8/theming/creating-automation-tools-for-custom-themes-gulpjs.
  // gulp.task('imagemin', function () {
  //   return gulp.src('./src/images/*')
  //     .pipe(imagemin({
  //       progressive: true,
  //       svgoPlugins: [{removeViewBox: false}],
  //       use: [pngquant()]
  //     }))
  //     .pipe(gulp.dest('./images'));
  // });

  gulp.task('sass', function () {
    var task = gulp.
      src(options.sass.src + '/**/*.scss');
    if (options.onError) {
      // Error management in gulp.
      task.pipe(plugins.plumber({errorHandler: options.onErrorCallback}));
    }
    task.pipe(plugins.sass({
        outputStyle: 'expanded',
        includePaths: options.sass.includePaths
      }))
      .pipe(plugins.postcss(options.postcssOptions))
      .pipe(plugins.rucksack())
      .pipe(plugins.stripCssComments())
      .pipe(gulp.dest(options.sass.dest));
    return task;
  });

  gulp.task('sass:dev', function () {
    var task = gulp
      .src(options.sass.src + '/**/*.scss');
    if (options.onError) {
      // Error management in gulp.
      task.pipe(plugins.plumber({errorHandler: options.onErrorCallback}));
    }
    task.pipe(plugins.sourcemaps.init())
      .pipe(plugins.sass({
        outputStyle: 'expanded',
        includePaths: options.sass.includePaths,
        sourceComments: 'map', sourceMap: 'sass'
      }))
      .pipe(plugins.postcss(options.postcssOptions))
      // A little bag of CSS superpowers http://simplaio.github.io/plugins.rucksack.
      .pipe(plugins.rucksack())
      .pipe(plugins.sourcemaps.write('.'))
      .pipe(gulp.dest(options.sass.dest));
    return task;
  });

  gulp.task('sass:lint', function () {
    return gulp.src(options.sass.src + '/**/*.scss')
      .pipe(plugins.postcss(options.processors, {syntax: plugins.syntax_scss}));
  });

  gulp.task('clean:css', function () {
    return plugins.del([
      'css/**/*'
    ]);
  });

  // Gulp run all tasks.

  gulp.task('default', ['clean', 'sass:lint', 'sass']);
  gulp.task('prod', ['default']);
  gulp.task('dev', ['sass:dev']);
  gulp.task('clean', ['clean:css']);

  // Gulp watches.

  gulp.task('watch', function () {
    gulp.watch(options.sass.src + '/**/*.scss', ['sass']);
  });
  gulp.task('watch:dev', function () {
    gulp.watch(options.sass.src + '/**/*.scss', ['sass:dev']);
  });

};
